package com.jenkins.demo.controller;

import com.jenkins.demo.model.User;
import com.jenkins.demo.service.UserService;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.DisplayName;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

/**
 * User Controller Integration Tests
 */
@WebMvcTest(UserController.class)
class UserControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private UserService userService;

    @Autowired
    private ObjectMapper objectMapper;

    private User testUser;
    private List<User> testUsers;

    @BeforeEach
    void setUp() {
        testUser = new User(1L, "testuser", "test@example.com", "测试用户", "ACTIVE");
        testUsers = Arrays.asList(
            testUser,
            new User(2L, "user2", "user2@example.com", "用户2", "ACTIVE")
        );
    }

    @Test
    @DisplayName("GET /api/users - 应该返回所有用户")
    void shouldReturnAllUsers() throws Exception {
        when(userService.getAllUsers()).thenReturn(testUsers);

        mockMvc.perform(get("/api/users"))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.length()").value(2))
                .andExpect(jsonPath("$[0].username").value("testuser"))
                .andExpect(jsonPath("$[1].username").value("user2"));

        verify(userService).getAllUsers();
    }

    @Test
    @DisplayName("GET /api/users/{id} - 应该返回指定用户")
    void shouldReturnUserById() throws Exception {
        when(userService.getUserById(1L)).thenReturn(Optional.of(testUser));

        mockMvc.perform(get("/api/users/1"))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.id").value(1))
                .andExpect(jsonPath("$.username").value("testuser"))
                .andExpect(jsonPath("$.email").value("test@example.com"));

        verify(userService).getUserById(1L);
    }

    @Test
    @DisplayName("GET /api/users/{id} - 用户不存在时应该返回404")
    void shouldReturn404WhenUserNotFound() throws Exception {
        when(userService.getUserById(999L)).thenReturn(Optional.empty());

        mockMvc.perform(get("/api/users/999"))
                .andExpect(status().isNotFound());

        verify(userService).getUserById(999L);
    }

    @Test
    @DisplayName("POST /api/users - 应该创建新用户")
    void shouldCreateNewUser() throws Exception {
        User newUser = new User(null, "newuser", "new@example.com", "新用户", null);
        User createdUser = new User(3L, "newuser", "new@example.com", "新用户", "ACTIVE");
        
        when(userService.createUser(any(User.class))).thenReturn(createdUser);

        mockMvc.perform(post("/api/users")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(newUser)))
                .andExpect(status().isCreated())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.id").value(3))
                .andExpect(jsonPath("$.username").value("newuser"))
                .andExpect(jsonPath("$.status").value("ACTIVE"));

        verify(userService).createUser(any(User.class));
    }

    @Test
    @DisplayName("POST /api/users - 创建用户时验证失败应该返回400")
    void shouldReturn400ForInvalidUserData() throws Exception {
        User invalidUser = new User(null, "", "invalid-email", "", null);

        mockMvc.perform(post("/api/users")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(invalidUser)))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DisplayName("PUT /api/users/{id} - 应该更新用户")
    void shouldUpdateUser() throws Exception {
        User updateData = new User(null, "updateduser", "updated@example.com", "更新用户", null);
        User updatedUser = new User(1L, "updateduser", "updated@example.com", "更新用户", "ACTIVE");
        
        when(userService.updateUser(eq(1L), any(User.class))).thenReturn(Optional.of(updatedUser));

        mockMvc.perform(put("/api/users/1")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(updateData)))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.username").value("updateduser"))
                .andExpect(jsonPath("$.email").value("updated@example.com"));

        verify(userService).updateUser(eq(1L), any(User.class));
    }

    @Test
    @DisplayName("DELETE /api/users/{id} - 应该删除用户")
    void shouldDeleteUser() throws Exception {
        when(userService.deleteUser(1L)).thenReturn(true);

        mockMvc.perform(delete("/api/users/1"))
                .andExpect(status().isNoContent());

        verify(userService).deleteUser(1L);
    }

    @Test
    @DisplayName("POST /api/users/{id}/activate - 应该激活用户")
    void shouldActivateUser() throws Exception {
        when(userService.activateUser(1L)).thenReturn(true);

        mockMvc.perform(post("/api/users/1/activate"))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.message").value("用户已激活"));

        verify(userService).activateUser(1L);
    }

    @Test
    @DisplayName("GET /api/users/stats - 应该返回用户统计信息")
    void shouldReturnUserStats() throws Exception {
        when(userService.getUserCount()).thenReturn(5L);

        mockMvc.perform(get("/api/users/stats"))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.totalUsers").value(5))
                .andExpect(jsonPath("$.timestamp").exists());

        verify(userService).getUserCount();
    }
}
